@@ -2,6 +2,7 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
+from narwhals._arrow.utils import ArrowExprNamespace
 from narwhals._expression_parsing import reuse_series_namespace_implementation
 
 if TYPE_CHECKING:
@@ -10,11 +11,8 @@ if TYPE_CHECKING:
     from narwhals._arrow.expr import ArrowExpr
 
 
-class ArrowExprCatNamespace:
-    def __init__(self: Self, expr: ArrowExpr) -> None:
-        self._compliant_expr = expr
-
+class ArrowExprCatNamespace(ArrowExprNamespace):
     def get_categories(self: Self) -> ArrowExpr:
         return reuse_series_namespace_implementation(
-            self._compliant_expr, "cat", "get_categories"
+            self.compliant, "cat", "get_categories"
         )
