@@ -2,6 +2,7 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
+from narwhals._arrow.utils import ArrowExprNamespace
 from narwhals._expression_parsing import reuse_series_namespace_implementation
 
 if TYPE_CHECKING:
@@ -10,9 +11,6 @@ if TYPE_CHECKING:
     from narwhals._arrow.expr import ArrowExpr
 
 
-class ArrowExprListNamespace:
-    def __init__(self: Self, expr: ArrowExpr) -> None:
-        self._expr = expr
-
+class ArrowExprListNamespace(ArrowExprNamespace):
     def len(self: Self) -> ArrowExpr:
-        return reuse_series_namespace_implementation(self._expr, "list", "len")
+        return reuse_series_namespace_implementation(self.compliant, "list", "len")
