@@ -1,10 +1,12 @@
 from __future__ import annotations
 
 import operator
+from functools import partial
 from functools import reduce
 from typing import TYPE_CHECKING
 from typing import Any
 from typing import Callable
+from typing import Container
 from typing import Iterable
 from typing import Literal
 from typing import Sequence
@@ -22,16 +24,21 @@ from narwhals._pandas_like.utils import extract_dataframe_comparand
 from narwhals._pandas_like.utils import horizontal_concat
 from narwhals._pandas_like.utils import vertical_concat
 from narwhals.typing import CompliantNamespace
+from narwhals.utils import exclude_column_names
+from narwhals.utils import get_column_names
 from narwhals.utils import import_dtypes_module
-from narwhals.utils import is_compliant_expr
+from narwhals.utils import passthrough_column_names
 
 if TYPE_CHECKING:
     from typing_extensions import Self
+    from typing_extensions import TypeAlias
 
     from narwhals.dtypes import DType
     from narwhals.utils import Implementation
     from narwhals.utils import Version
 
+    _Scalar: TypeAlias = Any
+
 
 class PandasLikeNamespace(CompliantNamespace[PandasLikeDataFrame, PandasLikeSeries]):
     @property
@@ -106,7 +113,17 @@ class PandasLikeNamespace(CompliantNamespace[PandasLikeDataFrame, PandasLikeSeri
     # --- selection ---
     def col(self: Self, *column_names: str) -> PandasLikeExpr:
         return PandasLikeExpr.from_column_names(
-            *column_names,
+            passthrough_column_names(column_names),
+            function_name="col",
+            implementation=self._implementation,
+            backend_version=self._backend_version,
+            version=self._version,
+        )
+
+    def exclude(self: Self, excluded_names: Container[str]) -> PandasLikeExpr:
+        return PandasLikeExpr.from_column_names(
+            partial(exclude_column_names, names=excluded_names),
+            function_name="exclude",
             implementation=self._implementation,
             backend_version=self._backend_version,
             version=self._version,
@@ -121,20 +138,9 @@ class PandasLikeNamespace(CompliantNamespace[PandasLikeDataFrame, PandasLikeSeri
         )
 
     def all(self: Self) -> PandasLikeExpr:
-        return PandasLikeExpr(
-            lambda df: [
-                PandasLikeSeries(
-                    df._native_frame[column_name],
-                    implementation=self._implementation,
-                    backend_version=self._backend_version,
-                    version=self._version,
-                )
-                for column_name in df.columns
-            ],
-            depth=0,
+        return PandasLikeExpr.from_column_names(
+            get_column_names,
             function_name="all",
-            evaluate_output_names=lambda df: df.columns,
-            alias_output_names=None,
             implementation=self._implementation,
             backend_version=self._backend_version,
             version=self._version,
@@ -405,16 +411,16 @@ class PandasWhen:
         condition: PandasLikeExpr,
         implementation: Implementation,
         backend_version: tuple[int, ...],
-        then_value: Any = None,
-        otherwise_value: Any = None,
+        then_value: PandasLikeExpr | _Scalar = None,
+        otherwise_value: PandasLikeExpr | _Scalar = None,
         *,
         version: Version,
     ) -> None:
         self._implementation = implementation
         self._backend_version = backend_version
-        self._condition = condition
-        self._then_value = then_value
-        self._otherwise_value = otherwise_value
+        self._condition: PandasLikeExpr = condition
+        self._then_value: PandasLikeExpr | _Scalar = then_value
+        self._otherwise_value: PandasLikeExpr | _Scalar = otherwise_value
         self._version = version
 
     def __call__(self: Self, df: PandasLikeDataFrame) -> Sequence[PandasLikeSeries]:
@@ -422,10 +428,9 @@ class PandasWhen:
         condition = self._condition(df)[0]
         condition_native = condition._native_series
 
-        if is_compliant_expr(self._then_value):
-            value_series: PandasLikeSeries = self._then_value(df)[0]
+        if isinstance(self._then_value, PandasLikeExpr):
+            value_series = self._then_value(df)[0]
         else:
-            # `self._then_value` is a scalar
             value_series = plx._create_series_from_scalar(
                 self._then_value, reference_series=condition.alias("literal")
             )
@@ -441,14 +446,13 @@ class PandasWhen:
                 )
             ]
 
-        if is_compliant_expr(self._otherwise_value):
-            otherwise_series: PandasLikeSeries = self._otherwise_value(df)[0]
+        if isinstance(self._otherwise_value, PandasLikeExpr):
+            otherwise_series = self._otherwise_value(df)[0]
         else:
-            # `self._then_value` is a scalar
-            otherwise_series = plx._create_series_from_scalar(
-                self._otherwise_value, reference_series=condition.alias("literal")
+            native_result = value_series_native.where(
+                condition_native, self._otherwise_value
             )
-            otherwise_series._broadcast = True
+            return [value_series._from_native_series(native_result)]
         otherwise_series_native = extract_dataframe_comparand(
             df._native_frame.index, otherwise_series
         )
@@ -458,7 +462,9 @@ class PandasWhen:
             )
         ]
 
-    def then(self: Self, value: PandasLikeExpr | PandasLikeSeries | Any) -> PandasThen:
+    def then(
+        self: Self, value: PandasLikeExpr | PandasLikeSeries | _Scalar
+    ) -> PandasThen:
         self._then_value = value
 
         return PandasThen(
@@ -492,7 +498,7 @@ class PandasThen(PandasLikeExpr):
         self._implementation = implementation
         self._backend_version = backend_version
         self._version = version
-        self._call = call
+        self._call: PandasWhen = call
         self._depth = depth
         self._function_name = function_name
         self._evaluate_output_names = evaluate_output_names
@@ -500,11 +506,8 @@ class PandasThen(PandasLikeExpr):
         self._call_kwargs = call_kwargs or {}
 
     def otherwise(
-        self: Self, value: PandasLikeExpr | PandasLikeSeries | Any
+        self: Self, value: PandasLikeExpr | PandasLikeSeries | _Scalar
     ) -> PandasLikeExpr:
-        # type ignore because we are setting the `_call` attribute to a
-        # callable object of type `PandasWhen`, base class has the attribute as
-        # only a `Callable`
-        self._call._otherwise_value = value  # type: ignore[attr-defined]
+        self._call._otherwise_value = value
         self._function_name = "whenotherwise"
         return self
